/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.mall.controller;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.mall.entity.BrandsInfo;
import org.springblade.modules.mall.service.IBrandsInfoService;
import org.springblade.modules.mall.vo.BrandsInfoVO;
import org.springblade.modules.mall.wrapper.BrandsInfoWrapper;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 品牌信息表 控制器
 *
 * @author BladeX
 * @since 2022-05-08
 */
@RestController
@AllArgsConstructor
@RequestMapping("mall/brandsinfo")
@Api(value = "品牌信息表", tags = "品牌信息表接口")
public class BrandsInfoController extends BladeController {

	private final IBrandsInfoService brandsInfoService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入brandsInfo")
	public R<BrandsInfoVO> detail(BrandsInfo brandsInfo) {
		BrandsInfo detail = brandsInfoService.getOne(Condition.getQueryWrapper(brandsInfo));
		return R.data(BrandsInfoWrapper.build().entityVO(detail));
	}

	/**
	 * 分页 品牌信息表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入brandsInfo")
	public R<IPage<BrandsInfoVO>> list(BrandsInfo brandsInfo, Query query) {
		QueryWrapper<BrandsInfo> queryWrapper = Condition.getQueryWrapper(brandsInfo);
		if (StrUtil.isNotBlank(brandsInfo.getBrandName())) {
			queryWrapper.like("brand_name", brandsInfo.getBrandName());
		}
		IPage<BrandsInfo> pages = brandsInfoService.page(Condition.getPage(query), queryWrapper);
		return R.data(BrandsInfoWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 品牌信息表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入brandsInfo")
	public R<IPage<BrandsInfoVO>> page(BrandsInfoVO brandsInfo, Query query) {
		IPage<BrandsInfoVO> pages = brandsInfoService.selectBrandsInfoPage(Condition.getPage(query), brandsInfo);
		return R.data(pages);
	}

	/**
	 * 新增 品牌信息表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入brandsInfo")
	public R save(@Valid @RequestBody BrandsInfo brandsInfo) {
		return R.status(brandsInfoService.save(brandsInfo));
	}

	/**
	 * 修改 品牌信息表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入brandsInfo")
	public R update(@Valid @RequestBody BrandsInfo brandsInfo) {
		return R.status(brandsInfoService.updateById(brandsInfo));
	}

	/**
	 * 新增或修改 品牌信息表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入brandsInfo")
	public R submit(@Valid @RequestBody BrandsInfo brandsInfo) {
		if (brandsInfo.getId() == null) {
			BrandsInfo detail = brandsInfoService.getOne(Wrappers.lambdaQuery(BrandsInfo.class)
				.eq(BrandsInfo::getBrandName, brandsInfo.getBrandName()));
			if (detail != null) {
				return R.fail("品牌已存在");
			}
		}
		return R.status(brandsInfoService.saveOrUpdate(brandsInfo));
	}


	/**
	 * 删除 品牌信息表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(brandsInfoService.removeByIds(Func.toLongList(ids)));
	}

	/**
	 * 根据名称查询品牌
	 */
	@GetMapping("/getByName")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "根据名称查询品牌", notes = "传入brandsInfo")
	public R<List<BrandsInfo>> getByName(BrandsInfo brandsInfo) {
		if (StrUtil.isBlank(brandsInfo.getBrandName())) {
			Query query = new Query();
			query.setCurrent(1);
			query.setSize(50);
			IPage<BrandsInfo> pages = brandsInfoService.page(Condition.getPage(query), Condition.getQueryWrapper(brandsInfo));
			return R.data(pages.getRecords());
		}
		if (NumberUtil.isNumber(brandsInfo.getBrandName()) && brandsInfo.getBrandName().length() == 19) {
			BrandsInfo info = brandsInfoService.getById(brandsInfo.getBrandName());
			if (info != null) {
				List<BrandsInfo> brandsInfoList = new ArrayList<>();
				brandsInfoList.add(info);
				return R.data(brandsInfoList);
			}
		}
		return R.data(brandsInfoService.lambdaQuery().eq(BrandsInfo::getId, brandsInfo.getBrandName())
			.or().like(BrandsInfo::getBrandName, brandsInfo.getBrandName()).list());
	}
}
