/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.mall.controller;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;

import javax.annotation.Resource;
import javax.validation.Valid;

import org.springblade.common.base.BaseEntity;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.service.IUserService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.mall.entity.Store;
import org.springblade.modules.mall.vo.StoreVO;
import org.springblade.modules.mall.wrapper.StoreWrapper;
import org.springblade.modules.mall.service.IStoreService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 商家列表 控制器
 *
 * @author BladeX
 * @since 2023-06-01
 */
@RestController
@AllArgsConstructor
@RequestMapping("mall/store")
@Api(value = "商家列表", tags = "商家列表接口")
public class StoreController extends BladeController {

    @Resource
	private IStoreService storeService;
    @Resource
    private IUserService userService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入store")
	public R<StoreVO> detail(Store store) {
		Store detail = storeService.getOne(Condition.getQueryWrapper(store));
		return R.data(StoreWrapper.build().entityVO(detail));
	}

	/**
	 * 分页 商家列表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入store")
	public R<IPage<StoreVO>> list(Store store, Query query) {
		LambdaQueryWrapper<Store> storeLambdaQueryWrapper = Wrappers.lambdaQuery(Store.class);
		if(StrUtil.isNotBlank(store.getName())){
			storeLambdaQueryWrapper.like(Store::getName, store.getName());
		}
		if(StrUtil.isNotBlank(store.getUserName())){
			storeLambdaQueryWrapper.like(Store::getUserName, store.getUserName());
		}
		if(ObjectUtil.isNotEmpty(store.getAuditState())){
			storeLambdaQueryWrapper.eq(Store::getAuditState, store.getAuditState());
		}
		if(ObjectUtil.isNotEmpty(store.getState())){
			storeLambdaQueryWrapper.eq(Store::getState, store.getState());
		}
		if(StrUtil.isEmpty(query.getAscs()) && StrUtil.isEmpty(query.getDescs())){
			storeLambdaQueryWrapper.orderByDesc(BaseEntity::getCreateTime);
		}
		IPage<Store> pages = storeService.page(Condition.getPage(query), storeLambdaQueryWrapper);
		return R.data(StoreWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 商家列表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入store")
	public R<IPage<StoreVO>> page(StoreVO store, Query query) {
		IPage<StoreVO> pages = storeService.selectStorePage(Condition.getPage(query), store);
		return R.data(pages);
	}

	/**
	 * 新增 商家列表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入store")
	public R save(@Valid @RequestBody Store store) {
		Store store1 = storeService.getOne(Wrappers.lambdaQuery(Store.class).eq(Store::getTel, store.getTel()));
		if(ObjectUtil.isNotEmpty(store1)){
			return R.fail("这个电话已经注册过供应商，无法重复注册");
		}
		User query1 = userService.getOne(Wrappers.<User>query().lambda().eq(User::getAccount, store.getTel()));
		if (query1 != null){
			return R.fail("该手机号已成为会员需要去会员中设置供应商！");
		}
		return R.status(storeService.addStore(store));
	}

	/**
	 * 修改 商家列表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入store")
	public R update(@Valid @RequestBody Store store) {
		return R.status(storeService.updateById(store));
	}

	/**
	 * 新增或修改 商家列表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入store")
	public R submit(@Valid @RequestBody Store store) {
		Store store1 = storeService.getOne(Wrappers.lambdaQuery(Store.class).eq(Store::getUserId, store.getUserId()));
		if(ObjectUtil.isNotEmpty(store1)){
			store.setId(store1.getId());
		}else{
			store.setState(1);
			store.setAuditState(1);
		}
		if(ObjectUtil.isNotEmpty(store.getAuditState()) && store.getAuditState() == 1 && store.getState() == 1){
			userService.setRoleByRoleName(store.getUserId(), "供应商");
		}else {
			if(ObjectUtil.isNotEmpty(store.getAuditState()) && (store.getAuditState() != 1 || store.getState() != 1)){
				userService.removeRoleByRoleName(store.getUserId(), "供应商");
			}
		}
		return R.status(storeService.setStore(store));
	}

	
	/**
	 * 删除 商家列表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "id", required = true) @RequestParam String id) {
		Store store = storeService.getById(id);
		storeService.remove(store);
		return R.success("删除成功");
	}

	
}
