/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.platform.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.platform.entity.Article;
import org.springblade.modules.platform.vo.ArticleVO;
import org.springblade.modules.platform.wrapper.ArticleWrapper;
import org.springblade.modules.platform.service.IArticleService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 文章 控制器
 *
 * @author BladeX
 * @since 2023-07-04
 */
@RestController
@AllArgsConstructor
@RequestMapping("platform/article")
@Api(value = "文章", tags = "文章接口")
public class ArticleController extends BladeController {

	private final IArticleService articleService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入article")
	public R<ArticleVO> detail(Article article) {
		Article detail = articleService.getOne(Condition.getQueryWrapper(article));
		return R.data(ArticleWrapper.build().entityVO(detail));
	}

	/**
	 * 分页 文章
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入article")
	public R<IPage<ArticleVO>> list(Article article, Query query) {
		QueryWrapper<Article> queryWrapper = Condition.getQueryWrapper(article);
		queryWrapper.select("id", "title", "create_time");
		IPage<Article> pages = articleService.page(Condition.getPage(query), queryWrapper);
		return R.data(ArticleWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 文章
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入article")
	public R<IPage<ArticleVO>> page(ArticleVO article, Query query) {
		IPage<ArticleVO> pages = articleService.selectArticlePage(Condition.getPage(query), article);
		return R.data(pages);
	}

	/**
	 * 新增 文章
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入article")
	public R save(@Valid @RequestBody Article article) {
		return R.status(articleService.save(article));
	}

	/**
	 * 修改 文章
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入article")
	public R update(@Valid @RequestBody Article article) {
		return R.status(articleService.updateById(article));
	}

	/**
	 * 新增或修改 文章
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入article")
	public R submit(@Valid @RequestBody Article article) {
		return R.status(articleService.saveOrUpdate(article));
	}

	
	/**
	 * 删除 文章
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(articleService.removeByIds(Func.toLongList(ids)));
	}

	
}
